
;Define Maximum software Buffer requirement here.
;Remember each buffer takes 14 bytes of GPRAM

#define MAX_TX_SOFT_BUFFER      0x03



;#define        CANIntLowPrior  ;Define if CAN Interrupt lower
                                ;priority is required.






; *********************************************************************
; *
; * CAN_CONFIG_FLAGS
; *
; * This values define flags related to configuring CAN module.
; * Routines CANInitializeFunc() and CANSetBaudRateFunc() uses these
; * codes. One or more these values may be ANDed to form multiple
; * flags.
; *
; ********************************************************************


CAN_CONFIG_DEFAULT:             = B'11111111'   ; 11111111

CAN_CONFIG_PHSEG2_PRG_BIT:      = B'10000000'
CAN_CONFIG_PHSEG2_PRG_BIT_NO:   = 0x07
CAN_CONFIG_PHSEG2_PRG_ON:       = B'11111111'   ; 1XXXXXXX
CAN_CONFIG_PHSEG2_PRG_OFF:      = B'01111111'   ; 0XXXXXXX

CAN_CONFIG_MSG_BITS:            = B'01100000'
CAN_CONFIG_ALL_MSG:             = B'11111111'   ; XX11XXXX
CAN_CONFIG_VALID_XTD_MSG:       = B'11011111'   ; X10XXXXX
CAN_CONFIG_VALID_STD_MSG:       = B'10111111'   ; X01XXXXX
CAN_CONFIG_ALL_VALID_MSG:       = B'10011111'   ; X00XXXXX

CAN_CONFIG_SAMPLE_BIT:          = B'00010000'
CAN_CONFIG_SAMPLE_BIT_NO:       = 0x04
CAN_CONFIG_SAMPLE_ONCE:         = B'11101111'   ; XXX1XXXX
CAN_CONFIG_SAMPLE_THRICE:       = B'11111111'   ; XXX0XXXX

CAN_CONFIG_MSG_TYPE_BIT:        = B'00001000'
CAN_CONFIG_MSG_TYPE_BIT_NO:     = 0x03
CAN_CONFIG_STD_MSG:             = B'11111111'   ; XXXX1XXX
CAN_CONFIG_XTD_MSG:             = B'11110111'   ; XXXX0XXX

CAN_CONFIG_DBL_BUFFER_BIT:      = B'00000100'
CAN_CONFIG_DBL_BUFFER_BIT_NO:   = 0x02
CAN_CONFIG_DBL_BUFFER_ON:       = B'11111111'   ; XXXXX1XX
CAN_CONFIG_DBL_BUFFER_OFF:      = B'11111011'   ; XXXXX0XX


CAN_CONFIG_LINE_FILTER_BIT:     = B'00000001'
CAN_CONFIG_LINE_FILTER_BIT_NO:  = 0x0
CAN_CONFIG_LINE_FILTER_ON:      = B'11111111'   ; XXXXXXX1
CAN_CONFIG_LINE_FILTER_OFF:     = B'11111110'   ; XXXXXXX0





; *********************************************************************
; *
; * CAN_OP_MODE
; *
; * This values define codes related to CAN module operation mode.
; * CANSetOperationModefunc() routine requires this code.
; * These values must be used by itself
; * i.e. it cannot be ANDed to form * multiple values.
; *
; ********************************************************************

CAN_OP_MODE_BITS:       = B'11100000'   ; Use this to access opmode
                                        ; bits
CAN_OP_MODE_NORMAL:     = B'00000000'
CAN_OP_MODE_SLEEP:      = B'00100000'
CAN_OP_MODE_LOOP:       = B'01000000'
CAN_OP_MODE_LISTEN:     = B'01100000'
CAN_OP_MODE_CONFIG:     = B'10000000'




;CAN Mask bits Registers
CAN_MASK_B0: =  0x0f18          ;Address of RXM0SIDH
CAN_MASK_B1: =  0x0f1c          ;Address of RXM1SIDH


;CAN FILTER Registers
CAN_FILTER_B0_F1: =     0x0f00  ;Address of RXF0SIDH
CAN_FILTER_B0_F2: =     0x0f04  ;Address of RXF1SIDH
CAN_FILTER_B1_F1: =     0x0f08  ;Address of RXF2SIDH
CAN_FILTER_B1_F2: =     0x0f0c  ;Address of RXF3SIDH
CAN_FILTER_B1_F3: =     0x0f10  ;Address of RXF4SIDH
CAN_FILTER_B1_F4: =     0x0f14  ;Address of RXF5SIDH



;CAN ID registers
CAN_TX_IDB0:    =       0xf41           ;Address of TXB0SIDH
CAN_TX_IDB1:    =       0xf31           ;Address of TXB1SIDH
CAN_TX_IDB2:    =       0xf21           ;Address of TXB2SIDH







; *********************************************************************
; *
; * CAN_TX_MSG_FLAGS
; *
; * This values define flags related to transmission of a
; * CAN message.  There could be more than one this flag
; * ANDed together to form multiple flags.
; *
; *********************************************************************

CAN_TX_PRIORITY_BITS:   = B'00000011'
CAN_TX_PRIORITY_0:      = B'11111100'   ;XXXXXX00
CAN_TX_PRIORITY_1:      = B'11111101'   ; XXXXXX01
CAN_TX_PRIORITY_2:      = B'11111110'   ; XXXXXX10
CAN_TX_PRIORITY_3:      = B'11111111'   ; XXXXXX11

CAN_TX_FRAME_BIT:       = B'00001000'
CAN_TX_STD_FRAME:       = B'11111111'   ; XXXXX1XX
CAN_TX_XTD_FRAME:       = B'11110111'   ; XXXXX0XX

CAN_TX_RTR_BIT:         = B'01000000'
CAN_TX_RTR_BIT_NO:      = 0x06
CAN_TX_NO_RTR_FRAME:    = B'11111111'   ; X1XXXXXX
CAN_TX_RTR_FRAME:       = B'10111111'   ; X0XXXXXX





; *********************************************************************
; *
; * CAN_RX_MSG_FLAGS
; *
; * These values define flags related to reception of a CAN
; * message.  There could be more than one this flag
; * ANDed together to form multiple flags.
; * If a particular bit is set, corresponding meaning is TRUE or else
; * it will be FALSE.
; *
; * e.g.
; *             if (MsgFlag & CAN_RX_OVERFLOW)
; *             {
; *                     // Receiver overflow has occurred.  We have lost ;
; *                     previous
; *                     // message.
; *                     ...
; *             }
; *
; *********************************************************************
CAN_RX_FILTER_BITS:     = B'00000111'   ;Use this to access filter bits
CAN_RX_FILTER_0:        = B'00000000'
CAN_RX_FILTER_1:        = B'00000001'
CAN_RX_FILTER_1_BIT_NO: = 0x00
CAN_RX_FILTER_2:        = B'00000010'
CAN_RX_FILTER_3:        = B'00000011'
CAN_RX_FILTER_4:        = B'00000100'
CAN_RX_FILTER_5:        = B'00000101'

CAN_RX_OVERFLOW:        = B'00001000'   ;Set if Overflowed else cleared
CAN_RX_OVERFLOW_BIT_NO: = 0x03

CAN_RX_INVALID_MSG:     = B'00010000'   ;Set if invalid else cleared
CAN_RX_INVALID_MSG_BIT_NO: = 0x04

CAN_RX_XTD_FRAME:       = B'00100000'   ;Set if XTD message else cleared
CAN_RX_XTD_FRAME_BIT_NO: = 0x05

CAN_RX_RTR_FRAME:       = B'01000000'   ;Set if RTR message else cleared
CAN_RX_RTR_FRAME_BIT_NO: = 0x06         ;RTR message bit

CAN_RX_DBL_BUFFERED: = B'10000000'      ;Set if this message was hardware
CAN_RX_DBL_BUFFERED_BIT_NO: = 0x07      ;double-buffered







;This Flag is used to determine which buffer has been loaded.
CANTxBuf0Flag:  = 0x00
CANTxBuf1Flag:  = 0x01
CANTxBuf2Flag:  = 0x02
CANTxSoftBufFlag: = 0x07



TxBufData: = D'14' * MAX_TX_SOFT_BUFFER ;Total RAM locations requirement
TxBufPtrSize: = (MAX_TX_SOFT_BUFFER + 3) * 0x02 ;
TxBufMaxSize: = MAX_TX_SOFT_BUFFER + 3 ;



;Used by CANInitialize, CANSetOperationMode and CANSetBaudRate function
                UDATA_OVR
m_SJW:          RES     01      ;SJW value
m_BRP:          RES     01      ;BRP value
m_PHSEG1:       RES     01      ;Phase Segment 1 value
m_PHSEG2:       RES     01      ;Phase Segment 2 value
m_PROPSEG2:     RES     01      ;Propagation Segment 1 value
m_Flags1:       RES     01      ;Configuration Flags
Reg1:           RES     04      ;32 bit register for intermediate storage
TempPtr:        RES     02
Temp32Data:     RES     08
TFSR0:          RES     02
TFSR1:          RES     02

m_DataLength:   RES     01      ;Tx message data length
m_RxFlags:
m_TxFlags:      RES     01










;****************************************************************************
;* MACRO SECTION
;****************************************************************************



;****************************************************************************
;* Macro:               SaveFSR0
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It copies the content of FSR0 in Temp. FSR0 reg.
;*
;****************************************************************************
SaveFSR0        macro
        movff   FSR0L,TFSR0
        movff   FSR0H,TFSR0+1
        endm

;****************************************************************************
;* Macro:               SaveFSR1
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It copies the content of FSR1 in Temp. FSR1 reg.
;*
;****************************************************************************
SaveFSR1        macro
        movff   FSR1L,TFSR1
        movff   FSR1H,TFSR1+1
        endm



;****************************************************************************
;* Macro:               RestoreFSR0
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It restores the content of FSR0 from Temp. FSR0 reg.
;*
;****************************************************************************
RestoreFSR0     macro
        movff   TFSR0,FSR0L
        movff   TFSR0+1,FSR0H
        endm

;****************************************************************************
;* Macro:               RestoreFSR1
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It restores the content of FSR1 from Temp. FSR1 reg.
;*
;****************************************************************************
RestoreFSR1     macro
        movff   TFSR1,FSR1L
        movff   TFSR1+1,FSR1H
        endm


;****************************************************************************
;* Macro:               EnableInt
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It enables the Global Interrupt
;*
;****************************************************************************
EnableInt       macro
        movlw   B'00011100'
        iorwf   PIE3            ;Enable Tx Int
        endm

;****************************************************************************
;* Macro:               DisableInt
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It Disables the Global Interrupt
;*
;****************************************************************************
DisableInt      macro
        movlw   B'11100011'
        andwf   PIE3            ;Disable Tx Int
        endm

;****************************************************************************
;* Macro:               EnablePeriphInt
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It enables the Peripheral Interrupt
;*
;****************************************************************************
EnablePeriphInt macro
        bsf     INTCON,PEIE
        endm

;****************************************************************************
;* Macro:               DisablePeriphInt
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;* Overview:            It disables the Peripheral Interrupt
;*
;****************************************************************************
DisablePeriphInt        macro
        bcf     INTCON,PEIE
        endm




;****************************************************************************
;* Macro:               CANAbortAll
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              None
;*
;* Side Effects:        None
;*
;****************************************************************************
CANAbortAll     macro
        bsf     CANCON,ABAT     ;Abort all pending transmissions
        endm



;****************************************************************************
;* Macro:               CANGetTxErrorCount
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              W reg = Current transmit error count as defined by
;*                              CAN specifications.
;*
;* Side Effects:        None
;*
;****************************************************************************
CANGetTxErrorCount      macro
        movf    TXERRCNT,W
        endm
        


;****************************************************************************
;* Macro:               CANGetRxErrorCount
;*
;* PreCondition:        None
;*
;* Input:               None
;*
;* Output:              W reg = Current receive error count as defined by
;*                              CAN specifications.
;*
;* Side Effects:        None
;*
;****************************************************************************
CANGetRxErrorCount      macro
        movf    RXERRCNT,w
        endm
        




;****************************************************************************
;* Macro:               CANInitialize   SJW, BRP,PHSEG1, PHSEG2, PROPSEG,
;*                                      CAN_CONFIG_FLAGS flags
;*
;* Input:               SJW - SJW value as defined in 18CXX8 data sheet
;*                      (Must be between 1 thru 4)
;*                      BRP - BRP value as defined in 18CXX8 data sheet
;*                      (Must be between 1 thru 64)
;*                      PHSEG1 - PHSEG1 value as defined in 18CXX8 data sheet
;*                      Must be between 1 thru 8)
;*                      PHSEG2 - PHSEG2 value as defined in 18CXX8 data sheet
;*                      (Must be between 1 thru 8)
;*                      PROPSEG - PROPSEG value as defined in 18CXX8 datasheet
;*                      (Must be between 1 thru 8)
;*                      flags - Value of type CAN_CONFIG_FLAGS
;*
;* Output:              CAN bit rate is set. All masks registers are set
;*                      '0' to allow all messages.
;*                      Filter registers are set according to flag value.
;*                      If (config & CAN_CONFIG_VALID_XTD_MSG)
;*                      Set all filters to XTD_MSG
;*                      Else if (config & CONFIG_VALID_STD_MSG)
;*                      Set all filters to STD_MSG
;*                      Else
;*                      Set half of the filters to STD while rests to
;*                      XTD_MSG.
;*
;* Side Effects:        All pending transmissions are aborted.
;*
;****************************************************************************
CANInitialize   macro   SJW, BRP, PHSEG1, PHSEG2, PROPSEG, Flags
        banksel m_SJW
        movlw   SJW
        movwf   m_SJW           ;Save SJW value

        banksel m_BRP
        movlw   BRP
        movwf   m_BRP           ;Save BRP value

        banksel m_PHSEG1
        movlw   PHSEG1
        movwf   m_PHSEG1        ;Save Phase Segment 1 value

        banksel m_PHSEG2
        movlw   PHSEG2
        movwf   m_PHSEG2        ;Save Phase Segment 2 value

        banksel m_PROPSEG2
        movlw   PROPSEG
        movwf   m_PROPSEG2      ;Save Propagation time value

        banksel m_Flags1        ;Save Configuration Flags info
        movlw   Flags
        movwf   m_Flags1

        call    CANInitializeFunc       ;Initialize the CAN module
        endm





;****************************************************************************
;* Macro:               CANSetBaudRate  SJW, BRP,PHSEG1, PHSEG2, PROPSEG,
;*                                      CAN_CONFIG_FLAGS flags
;*
;* PreCondition:        MCU must be in Configuration mode or else these
;*                      values will be ignored.
;*
;* Input:               SJW - SJW value as defined in 18CXX8 data sheet
;*                      (Must be between 1 thru 4)
;*                      BRP - BRP value as defined in 18CXX8 data sheet
;*                      (Must be between 1 thru 64)
;*                      PHSEG1 - PHSEG1 value as defined in 18CXX8 data sheet
;*                      Must be between 1 thru 8)
;*                      PHSEG2 - PHSEG2 value as defined in 18CXX8 data sheet
;*                      (Must be between 1 thru 8)
;*                      PROPSEG - PROPSEG value as defined in 18CXX8 datasheet
;*                      (Must be between 1 thru 8)
;*                      flags - Value of type CAN_CONFIG_FLAGS
;*
;* Output:              CAN bit rate is set. All masks registers are set
;*                      '0' to allow all messages.
;*                      Filter registers are set according to flag value.
;*                      If (config & CAN_CONFIG_VALID_XTD_MSG)
;*                      Set all filters to XTD_MSG
;*                      Else if (config & CONFIG_VALID_STD_MSG)
;*                      Set all filters to STD_MSG
;*                      Else
;*                      Set half of the filters to STD while rests to
;*                      XTD_MSG.
;*
;* Side Effects:        All pending transmissions are aborted.
;*
;****************************************************************************
CANSetBaudRate  macro   SJW, BRP, PHSEG1, PHSEG2, PROPSEG, Flags
        banksel m_SJW
        movlw   SJW
        movwf   m_SJW           ;Save SJW value

        banksel m_BRP
        movlw   BRP
        movwf   m_BRP           ;Save BRP value

        banksel m_PHSEG1
        movlw   PHSEG1
        movwf   m_PHSEG1        ;Save Phase Segment 1 value

        banksel m_PHSEG2
        movlw   PHSEG2
        movwf   m_PHSEG2        ;Save Phase Segment 2 value

        banksel m_PROPSEG2
        movlw   PROPSEG
        movwf   m_PROPSEG2      ;Save Propagation time value

        banksel m_Flags1        ;Save Configuration Flags info
        movlw   Flags
        movwf   m_Flags1

        call    CANSetBaudRateFunc      ;Initialize the Baud Rate
        endm







;****************************************************************************
;* Macro:               CANSetOperationMode     CAN_OP_MODE mode
;*
;* PreCondition:        None
;*
;* Input:               Value of type CAN_OP_MODE
;*
;*
;* Output:              MCU is set to requested mode
;*
;* Side Effects:        Databank is changed
;*
;* Overview:            Given mode byte is copied to CANSTAT and made
;*                      sure that requested mode is set.
;*
;* Note:                This is a blocking call.  It will not return until
;*                      requested mode is set.
;****************************************************************************
CANSetOperationMode macro OpMode
        movlw   OpMode
        call    CANSetOperationModeFunc ;Set CAN module Operation mode
        endm



;****************************************************************************
;* Macro:               CANSetReg       RegAddr,
;*                                      val,
;*                                      CAN_CONFIG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               RegAddr Starting address of a 32-bit buffer to be
;*                      updated
;*                      val - 32-bit value to be converted
;*                      Flags - Type of message - either
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are updated
;*                      If given id is of type extended identifier,
;*                      bits val<17:0> is copied to EIDH, EIDL and SIDH<1:0>
;*                      bits val<28:18> is copied to SIDH and SIDL
;*
;****************************************************************************
CANSetReg macro RegAddr, val, Flags
        banksel m_Flags1        ;Save Configuration Flags info
        movlw   Flags
        movwf   m_Flags1

        movlw   low(RegAddr)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(RegAddr)
        movwf   FSR0H
        movlw   low(val)        ;Store the 32-bit variable in register
        banksel Reg1
        movwf   Reg1            ;First LL byte
        movlw   high(val)
        movwf   Reg1+1          ;Second LH byte
        movlw   upper(val)
        movwf   Reg1+2          ;Third HL byte
val1 = val >> 8
        movlw   upper(val1)
        movwf   Reg1+3          ;Fourth HH byte
        call    CANSetRegFunc   ;Copy 32-bit value into required registers.
        endm







;****************************************************************************
;* Macro:               CANSetReg_IF    RegAddr,
;*                                      val,
;*                                      CAN_CONFIG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               RegAddr - Starting address of a 32-bit buffer to be
;*                      updated
;*                      val - 32-bit value to be converted
;*                      Flags - Address of Register containing flags info
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are updated
;*                      If given id is of type extended identifier,
;*                      bits val<17:0> is copied to EIDH, EIDL and SIDH<1:0>
;*                      bits val<28:18> is copied to SIDH and SIDL
;*
;****************************************************************************
CANSetReg_IF macro RegAddr, val, FlagsReg
        movlw   low(FlagsReg)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(FlagsReg)
        movwf   FSR0H
        movff   INDF0,m_Flags1

        movlw   low(RegAddr)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(RegAddr)
        movwf   FSR0H
        movlw   low(val)        ;Store the 32-bit variable in register
        banksel Reg1
        movwf   Reg1            ;First LL byte
        movlw   high(val)
        movwf   Reg1+1          ;Second LH byte
        movlw   upper(val)
        movwf   Reg1+2          ;Third HL byte
val1 = val >> 8
        movlw   upper(val1)
        movwf   Reg1+3          ;Fourth HH byte
        call    CANSetRegFunc   ;Copy 32-bit value into required registers.
        endm





;****************************************************************************
;* Macro:               CANSetReg_IV    RegAddr,
;*                                      val,
;*                                      CAN_CONFIG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               RegAddr - Starting address of a 32-bit buffer to be
;*                      updated
;*                      val - Starting address of 32 bit value to be converted
;*                              (LL:LH:HL:HH) format (Low -> High)
;*                      Flags - Type of message - either
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are updated
;*                      If given id is of type extended identifier,
;*                      bits val<17:0> is copied to EIDH, EIDL and SIDH<1:0>
;*                      bits val<28:18> is copied to SIDH and SIDL
;*
;****************************************************************************
CANSetReg_IV macro RegAddr, Var, Flags
        banksel m_Flags1        ;Save Configuration Flags info
        movlw   Flags
        movwf   m_Flags1

        movlw   low(Var)
        movwf   FSR0L
        movlw   high(Var)
        movwf   FSR0H
        movff   POSTINC0,Reg1   ;Save LL byte
        movff   POSTINC0,Reg1+1 ;save LH byte
        movff   POSTINC0,Reg1+2 ;save HL byte
        movff   POSTINC0,Reg1+3 ;save HH byte

        movlw   low(RegAddr)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(RegAddr)
        movwf   FSR0H
        call    CANSetRegFunc   ;copy 32-bit value into required registers.
        endm








;****************************************************************************
;* Macro:               CANSetReg_IV_IF RegAddr,
;*                                      var,
;*                                      CAN_CONFIG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               RegAddr - Starting address of a 32-bit buffer to be
;*                      updated
;*                      var - Starting address of 32 bit value to be converted
;*                              (LL:LH:HL:HH) format(Low -> High)
;*                      Flags - Address of Register containing flags info
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are updated
;*                      If given id is of type extended identifier,
;*                      bits val<17:0> is copied to EIDH, EIDL and SIDH<1:0>
;*                      bits val<28:18> is copied to SIDH and SIDL
;*
;****************************************************************************
CANSetReg_IV_IF macro RegAddr, Var, FlagsReg
        movlw   low(FlagsReg)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(FlagsReg)
        movwf   FSR0H
        movff   INDF0,m_Flags1

        movlw   low(Var)
        movwf   FSR0L
        movlw   high(Var)
        movwf   FSR0H
        movff   POSTINC0,Reg1   ;Save LL byte
        movff   POSTINC0,Reg1+1 ;save LH byte
        movff   POSTINC0,Reg1+2 ;save HL byte
        movff   POSTINC0,Reg1+3 ;save HH byte

        movlw   low(RegAddr)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(RegAddr)
        movwf   FSR0H
        call    CANSetRegFunc   ;copy 32-bit value into required registers.
        endm









;****************************************************************************
;* Macro:               CANSetReg_PREG_IV_IF    val,
;*                                              CAN_CONFIG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               FSR0: Starting address of a 32-bit buffer to be
;*                      updated
;*                      Var - Starting address of 32 bit value to be
;*                              converted, (LL:LH:HL:HH) format (Low -> High)
;*                      Flags - Address of Register containing flags info
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are updated
;*                      If given id is of type extended identifier,
;*                      bits val<17:0> is copied to EIDH, EIDL and SIDH<1:0>
;*                      bits val<28:18> is copied to SIDH and SIDL
;*
;****************************************************************************
CANSetReg_PREG_IV_IF macro Var, FlagsReg
        movlw   low(FlagsReg)
        movwf   FSR1L           ;Save the address of destination register
        movlw   high(FlagsReg)
        movwf   FSR1H
        movff   INDF1,m_Flags1

        movlw   low(Var)
        movwf   FSR1L
        movlw   high(Var)
        movwf   FSR1H
        movff   POSTINC1,Reg1   ;Save LL byte
        movff   POSTINC1,Reg1+1 ;save LH byte
        movff   POSTINC1,Reg1+2 ;save HL byte
        movff   POSTINC1,Reg1+3 ;save HH byte

        call    CANSetRegFunc   ;copy 32-bit value into required registers.
        endm






;****************************************************************************
;* Macro:               CANSetReg_PREG_DV_IF    FlagsReg
;*
;* PreCondition:        None
;*
;* Input:               FSR0: Starting address of a 32-bit buffer to be
;*                      updated
;*                      Reg1 - Starting address of 32 bit value to be
;*                              converted, (LL:LH:HL:HH) format (Low -> High)
;*                      Flags - Address of Register containing flags info
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are updated
;*                      If given id is of type extended identifier,
;*                      bits val<17:0> is copied to EIDH, EIDL and SIDH<1:0>
;*                      bits val<28:18> is copied to SIDH and SIDL
;*
;****************************************************************************
CANSetReg_PREG_DV_IF macro FlagsReg

        movlw   low(FlagsReg)
        movwf   FSR1L           ;Save the address of destination register
        movlw   high(FlagsReg)
        movwf   FSR1H
        movff   INDF1,m_Flags1
        call    CANSetRegFunc   ;copy 32-bit value into required registers.
        endm








;****************************************************************************
;* Macro:               CANReadReg      RegAddr,
;*                                      Val,
;*
;* PreCondition:        None
;*
;* Input:               RegAddr - Starting address of a 32-bit buffer to be
;*                      read
;*                      Val - Starting address of a buffer for storage of
;*                      converted 32-bit variable
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are read
;*                      If given id is of type extended identifier,
;*                      EIDL and EIDH are read too.
;*
;****************************************************************************
CANReadReg macro RegAddr, Val

        movlw   low(RegAddr)
        movwf   FSR0L           ;Save the address of destination register
        movlw   high(RegAddr)
        movwf   FSR0H

        movlw   low(Val)
        movwf   FSR1L           ;Save the address of destination register
        movlw   high(Val)
        movwf   FSR1H
        call    CANReadRegFunc
        endm








;****************************************************************************
;* Macro:               CANReadReg_PREG Val,
;*
;* PreCondition:        None
;*
;* Input:               FSR0 - Starting address of a 32-bit buffer to be
;*                      read
;*                      Val - Starting address of a buffer for storage of
;*                      converted 32-bit variable
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are read
;*                      If given id is of type extended identifier,
;*                      EIDL and EIDH are read too.
;*
;****************************************************************************
CANReadReg_PREG macro Val

        movlw   low(Val)
        movwf   FSR1L           ;Save the address of destination register
        movlw   high(Val)
        movwf   FSR1H
        call    CANReadRegFunc
        endm











;****************************************************************************
;* Macro:               CANReadReg_PPREG        RegAddr,
;*                                              Val,
;*
;* PreCondition:        None
;*
;* Input:               FSR0 - FSR0 contains an address of memory location
;*                      which contains address of starting address of buffer
;*                      to be read.
;*                      Val - Starting address of a buffer for storage of
;*                      converted 32-bit variable
;*
;* Output:              Given CAN id value 'val' is bit adjusted and copied
;*                      into corresponding PIC18CXX8 CAN registers
;*
;* Side Effects:        None
;*
;* Overview:            If given id is of type standard identifier,
;*                      only SIDH and SIDL are read
;*                      If given id is of type extended identifier,
;*                      EIDL and EIDH are read too.
;*
;****************************************************************************
CANReadReg_PPREG macro Val

        movff   POSTINC0,Temp1  ;Get the Low byte of Address pointer
        movff   POSTINC0,Temp2  ;Get the high byte of address pointer

        movff   Temp1,FSR0L     ;Point towards actual register
        movff   Temp2,FSR0H     ;

        movlw   low(Val)
        movwf   FSR1L           ;Save the address of destination register
        movlw   high(Val)
        movwf   FSR1H
        call    CANReadRegFunc
        endm









;****************************************************************************
;* Macro:               CANSendMessage  msgID,
;*                                      DataPtr,
;*                                      DataLngth,
;*                                      CAN_TX_MSG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               msgID - 32-bit Message ID
;*                      DataPtr - Starting address of data to be transmitted
;*                      DataLngth - Data Length (Must be between 0 to 8)
;*                      type - Type of message -CAN_TX_MSG_FLAGS type
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG &
;*                              CAN_TX_RTR_FRAME or CAN_TX_NO_RTR_FRAME
;*
;* Output:              Given CAN id value and data is copied into Tx buffer
;*                      according to priority for Transmission.
;*
;* Side Effects:        None
;*
;* Overview:            It copies the data in available hardware or software
;*                      buffer. If present data is of higher priority then
;*                      the data in hardware buffer then it aborts
;*                      transmission of lowest priority data in HW buffer and
;*                      copies it to SW buffer and copies present data to HW
;*                      Buffer for immediate transmission
;*
;****************************************************************************
CANSendMessage  macro   msgID, DataPtr, DataLngth, Flags
        movlw   low(msgID)      ;Store the 32-bit variable in register
        banksel Reg1
        movwf   Reg1            ;First LL byte
        movlw   high(msgID)
        movwf   Reg1+1          ;Second LH byte
        movlw   upper(msgID)
        movwf   Reg1+2          ;Third HL byte
msgID1 = msgID >> 8
        movlw   upper(msgID1)
        movwf   Reg1+3          ;Fourth HH byte

        movlw   low(DataPtr)
        movwf   FSR1L
        movlw   high(DataPtr)
        movwf   FSR1H

        movlw   DataLngth
        banksel m_DataLength
        movwf   m_DataLength

        movlw   Flags
        banksel m_TxFlags
        movwf   m_TxFlags
        call    CANSendMessageFunc
        endm







;****************************************************************************
;* Macro:               CANSendMessage_IID_IDL_IF       msgIDPtr,
;*                                                      DataPtr,
;*                                                      DataLngthPtr,
;*                                                      FlagsReg
;*
;* PreCondition:        None
;*
;* Input:               msgIDPtr - Starting address of 32-bit Message ID
;*                                 storage, format (LL:LH:HL:HH),(Low ->High)
;*                      DataPtr - Starting address of data to be transmitted
;*                      DataLngthPtr - Address of Register for Data Length
;*                                      storage
;*                      FlagsReg - Address of Register containing flags info
;*                              Type of message -CAN_TX_MSG_FLAGS type
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG &
;*                              CAN_TX_RTR_FRAME or CAN_TX_NO_RTR_FRAME
;*
;* Output:              Given CAN id value and data is copied into Tx buffer
;*                      according to priority for Transmission.
;*
;* Side Effects:        None
;*
;* Overview:            It copies the data in available hardware or software
;*                      buffer. If present data is of higher priority then
;*                      the data in hardware buffer then it aborts
;*                      transmission of lowest priority data in HW buffer and
;*                      copies it to SW buffer and copies present data to HW
;*                      Buffer for immediate transmission
;*
;****************************************************************************
CANSendMessage_IID_IDL_IF macro msgIDPtr, DataPtr, DataLngthPtr, FlagsReg

        movlw   low(msgIDPtr)
        movwf   FSR1L
        movlw   high(msgIDPtr)
        movwf   FSR1H

        movff   POSTINC1,Reg1           ;Copy Message ID value into Reg1
        movff   POSTINC1,Reg1+1         ;32-bit Source is Pointed by FSR1
        movff   POSTINC1,Reg1+2         ;32 bit storage Low -> High
        movff   POSTINC1,Reg1+3

        movlw   low(DataLngthPtr)
        movwf   FSR1L           ;Copy Data Length value into m_DataLength
        movlw   high(DataLngthPtr)
        movwf   FSR1H
        movff   INDF1,m_DataLength

        movlw   low(FlagsReg)
        movwf   FSR1L           ;Save the address of destination register
        movlw   high(FlagsReg)
        movwf   FSR1H
        movff   INDF1,m_TxFlags

        movlw   low(DataPtr)
        movwf   FSR1L
        movlw   high(DataPtr)
        movwf   FSR1H

        call    CANSendMessageFunc
        endm







;****************************************************************************
;* Macro:               CANReadMessage  msgIDPtr,
;*                                      DataPtr,
;*                                      DataLngth,
;*                                      CAN_RX_MSG_FLAGS type
;*
;* PreCondition:        None
;*
;* Input:               msgIDPtr - Starting address of 32-bit Message ID
;*                                      storage
;*                      DataPtr - Starting address for recd. data storage
;*                      DataLngth - Data Length (Must be between 0 to 8)
;*                      type - Type of message - either
;*                              CAN_CONFIG_XTD_MSG or CAN_CONFIG_STD_MSG &
;*                              CAN_TX_RTR_FRAME or CAN_TX_NO_RTR_FRAME
;*
;* Output:              If any pending received data is available then data
;*                      and ID values is returned
;*
;* Side Effects:        None
;*
;* Overview:
;*
;*
;*
;*
;*
;****************************************************************************
CANReadMessage  macro   msgIDPtr, DataPtr, DataLngth, Flags
        movlw   low(DataPtr)
        movwf   FSR0L
        movlw   high(DataPtr)
        movwf   FSR0H

        call    CANReadMessageFunc
        movwf   Temp1           ;Save return value

        movlw   low(msgIDPtr)
        movwf   FSR0L
        movlw   high(msgIDPtr)
        movwf   FSR0H
        movff   Temp32Data,POSTINC0     ;Transfer Message ID value
        movff   Temp32Data+1,POSTINC0
        movff   Temp32Data+2,POSTINC0
        movff   Temp32Data+3,POSTINC0

        movlw   low(DataLngth)
        movwf   FSR0L
        movlw   high(DataLngth) ;Transfer Data Length value
        movwf   FSR0H
        movff   m_DataLength,INDF0

        movlw   low(Flags)
        movwf   FSR0L
        movlw   high(Flags)     ;Transfer Flags value
        movwf   FSR0H
        movff   m_RxFlags,INDF0

        movf    Temp1,W         ;Restore return value
        endm








